import java.io.*;
import java.util.*;
import java.lang.System;
import org.xml.sax.*;
import java.lang.InstantiationException;
import org.xml.sax.*;
import org.w3c.dom.*;
import org.xml.sax.helpers.XMLReaderFactory;


/*
 * A class which can create a Keynote slideshow with a basic graph illustrating
 * how to dynamically create a presentation.
 * 
 */
public class GraphFiller implements ContentHandler {

	private Locator locator;
	private Map namespaceMappings;
	private PrintWriter writer;
	private	StringBuffer buffer;	// prints the 
	private StringBuffer ns;
	
	public GraphFiller() {

		super();				
		buffer = new StringBuffer();
		ns = new StringBuffer();		
	}
	
	/*	Creates a presentation by performing a straight copy of an existing template
	 */
	public static void createPresentation(File template, File bundle) throws IOException {

		if (bundle.exists()) {		
			recursiveDelete(bundle);
		}	

		recursiveCopy(template, bundle);
	}

	/*	Recursively copies an existing file into a target file. The destination
	 *  file can't exist.
	 */	
	public static void recursiveCopy(File og, File dest) throws IOException {

		if (og.isDirectory()) {	// the original file is a directory
		
			if (dest.mkdir()) {	// make the new directory for the copy
			
				File[] listing = og.listFiles();	// get a listing of all the names of files in this directory

				// loop through all the files and copy them into the target directory
				for (int i = 0; i < listing.length; i++) {
					recursiveCopy(listing[i], new File(dest, listing[i].getName()));
				}
				
			} else {	// the target directory wasn't created successfully, so abort
				throw new IOException("Can't make directory " + dest.toString());
			}

		} else { 		// the original file is not a directory (ie: it's a regular file)
			
			// read the file in chunks and send it to the new file's stream
			FileInputStream streamIn  = new FileInputStream(og);
			FileOutputStream streamOut = new FileOutputStream(new File(dest.getParent(), og.getName()));
			// 16k is likely to be a multiple of the file block size.
			byte[] buffer = new byte[16 * 1024];
			
			int readBytes = 0;			
			while((readBytes=streamIn.read(buffer)) != -1) {
				streamOut.write(buffer, 0, readBytes);
			}
			streamIn.close();
			streamOut.close();
		}
	}
	
	/*	Deletes a file. If the file is a directory, it will first delete all the
	 *	contents of the directory, and then delete it.
	 */
	public static void recursiveDelete(File arg) throws IOException {

		// is this file a directory?
		// if so, we'll have to be sure to empty its contents first
		if (arg.isDirectory()) {
			// get the list of filenames in the directory
			String[] files = arg.list();
			// if the directory is empty, then attempt to delete it
			if (files.length == 0) {
				if (! arg.delete()) {
					throw new IOException ("Can't delete " + arg);
				}			
			// the directory isn't empty, so we'll have to recursively delete its contents
			} else {
				// loop through the directory contents, and recursively call this function to empty it
				int i = 0;
				while (i < files.length) {
					recursiveDelete(new File(arg, files[i]));
					i++;
				}
				// now that the directory is empty, delete the directory itself
				if (! arg.delete()) {
					throw new IOException ("can't delete " + arg);
				}
			}
		// the file is NOT a directory, so we should be able to delete it
		} else {
			if (! arg.delete()) {
				throw new IOException("Can't delete " + arg);
			} 
		}
	}
		
	public void endDocument() throws SAXException {

		writer.println("");
		writer.flush();
		writer.close();
	}
	
	/*
	 *	Appends the namespace information to a buffer to be sent to the print stream in the startElement method.
	 */
	public void startPrefixMapping(String prefix, String uri) throws SAXException {
		
		namespaceMappings.put(prefix, uri);
		ns.append(" xmlns" + (prefix.equals("") ? "" : ":" + prefix) + "=\"" + uri + "\"");
	}

	/*	
	 *
	 */
	public void endPrefixMapping(String prefix) throws SAXException {		
	
		String uri = null;
		String thisPrefix = null;
	
		for (Iterator i = namespaceMappings.keySet().iterator(); i.hasNext(); ) {
			
			uri = (String) i.next();
			thisPrefix = (String) namespaceMappings.get(uri);
						
			if (prefix.equals(thisPrefix)) {
				namespaceMappings.remove(uri);
				break;
			}
		}
	}

	/*	
	 *
	 */	
	public void setDocumentLocator(Locator locator) {
		this.locator = locator;
	}
	
	/*	
	 *
	 */	
	public void startDocument() throws SAXException { 

		namespaceMappings = new HashMap();
		
		try {
			writer = new PrintWriter(new FileOutputStream(new File(new File ("graph.key"), "presentation.apxl")));
		} catch(FileNotFoundException e) {
			throw new SAXException("Can't create 'graph.key'");
		}
		
		writer.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
	}
	
	/* Send the characters to the writer. */
	public void characters(char ch[], int start, int length) throws SAXException {
		writer.print(new String(ch, start, length));	
	}
	
	/*
	 *
	 *
	 */
	public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {

		buffer = new StringBuffer();

		// if the name of the element is NOT the key we're looking for, then simply send it to the printer with
		// its attribute and namespace nodes (if any).
		if (! localName.equals("graphFiller")) {

			buffer.append("<").append(qName);

			for (int i = 0; i < atts.getLength(); i++) {
				buffer.append(" ").append(atts.getQName(i)).append("=").append("\"").append(atts.getValue(i)).append("\"");
			} 

			if (! ns.toString().equals("")) {
				buffer.append(ns.toString());
			} 

			buffer.append(">");
			writer.print(buffer.toString());

		// we've now come across the element that we're looking for, so we'll replace it with the information
		// for the chart, which we'll simply take out of a pre-populated HashMap, but it
		// is trivial to change it to another, more interesting data source (such as 
		// a database or another XML document)
		} else {

			try {

				// create  a sample datastore to populate the chart with
				ArrayList list = new ArrayList();
				list.add(new Integer(87));	
				list.add(new Integer(45));
				list.add(new Integer(63));
				list.add(new Integer(71));
				
				String key;
				for (int i = 0; i < list.size(); i++) {

					key = "Quarter " + (i + 1);
					int value = ((Integer)list.get(i)).intValue();
					System.out.println(key + " : " + value);

					buffer.append("<array>\n");
					buffer.append("<string>").append(key).append("</string>\n<number>").append("" + value).append("</number>\n");
					buffer.append("</array>\n");
				}

				writer.print(buffer.toString());
			
			} catch (Exception e) { 
				throw new SAXException(e);
			} 

		}
		
		ns = new StringBuffer();
	}
	
	public void endElement(String namespaceURI, String localName, String qName) throws SAXException {

		// if the element is not the key that we're looking for, send it to the printer
		// if it is, we don't want to do anything
		if (! localName.equals("graphFiller")) {
			writer.print("</" + qName + ">");	
		}
	}
	
	public void processingInstruction(String target, String data) throws SAXException {
		// do nothing	
	}
	
	public void ignorableWhitespace(char ch[], int start, int length) throws SAXException {
		// do nothing
	}
	
	public void skippedEntity(String name) throws SAXException {
		// do nothing
	}

	public static void main(String[] args) {
	
		if (args.length != 1) {
			System.out.println("Usage: java GraphFiller <template file>");
			System.exit(0);	
		}
	
		try {			
		
			createPresentation(new File(args[0]), new File("graph.key"));
			InputSource inputSource = new InputSource(new java.io.FileInputStream(new java.io.File(args[0], "presentation.apxl")));
			XMLReader reader = XMLReaderFactory.createXMLReader("org.apache.xerces.parsers.SAXParser");
			
			reader.setContentHandler(new GraphFiller());
			reader.parse(inputSource);
			
		} catch(FileNotFoundException e) {
			System.out.println("File " + args[0] + " can not be found.");
		} catch(IOException e) {
			System.out.println(e.getMessage());
		} catch(SAXException e) {
			System.out.println("Error parsing " + args[0] + ": " + e.getMessage());
		}
	}
} 
